#version 400 core

in vec3 pass_tangent;
in vec3 bitangent;
in vec2 pass_textureCoords;
in vec3 surfaceNormal;
in vec3 toLightVector[4];
in vec3 toCameraVector;
in float visibility;

out vec4 out_Color;

vec3 mappedNormal;
float normalMapStrength = 1.5;

uniform sampler2D modelTexture;
uniform vec3 lightColour[4];
uniform vec3 attenuation[4];
uniform float shineDamper;
uniform float reflectivity;
uniform vec3 skyColour;

uniform bool hasNormalMap = false;
uniform sampler2D normalMap;

void main(void) {

	if(hasNormalMap == true) {
	
		mappedNormal = (2 * texture2D(normalMap, pass_textureCoords).xyz - 1);

        //apply strength
        mappedNormal.z -= 1;
        mappedNormal *= normalMapStrength;
        mappedNormal.z += 1;

        //transform to world space
        mappedNormal *= transpose(mat3(pass_tangent, bitangent, surfaceNormal));
	
	} else {

		mappedNormal = surfaceNormal;

	}

	//vec3 unitNormal = mappedNormal;
	vec3 unitNormal = normalize(mappedNormal);
	vec3 unitVectorToCamera = normalize(toCameraVector);

	vec3 totalDiffuse = vec3(0.0);
	vec3 totalSpecular = vec3(0.0);

	for(int i = 0; i < 4; i++) {

		float distance = length(toLightVector[i]);
		float attFactor = attenuation[i].x + (attenuation[i].y * distance) + (attenuation[i].z * distance * distance);
		vec3 unitLightVector = normalize(toLightVector[i]);
		float nDotl = dot(unitNormal, unitLightVector);
		float brightness = max(nDotl, 0.0);
		vec3 lightDirection = -unitLightVector;
		vec3 reflectedLightDirection = reflect(lightDirection, unitNormal);
		float specularFactor = dot(reflectedLightDirection, unitVectorToCamera);
		specularFactor = max(specularFactor, 0.0);
		float dampedFactor = pow(specularFactor, shineDamper);
		totalDiffuse = totalDiffuse + (brightness * lightColour[i]) / attFactor;
		totalSpecular = totalSpecular + (dampedFactor * reflectivity * lightColour[i]) / attFactor;

	}

	totalDiffuse = max(totalDiffuse, 0.2);

	vec4 textureColour = texture(modelTexture, pass_textureCoords);

	if(textureColour.a < 0.01) {
	
		discard;

	}

	out_Color = vec4(totalDiffuse, 1.0) * textureColour + vec4(totalSpecular, 0.0);
	out_Color = vec4(mix(skyColour, out_Color.xyz, visibility), out_Color.a);
}