#version 400 core

in vec3 position;
in vec2 textureCoords;
in vec3 normal;
in vec3 tangent;

out vec3 pass_tangent;
out vec3 bitangent;
out vec2 pass_textureCoords;
out vec3 surfaceNormal;
out vec3 toLightVector[4];
out vec3 toCameraVector;
out float visibility;

uniform mat4 transformationMatrix;
uniform mat4 projectionMatrix;
uniform mat4 viewMatrix;
uniform vec3 lightPosition[4];

uniform float useFakeLighting;

uniform float numberOfRows;
uniform vec2 offset;

uniform vec4 clipPlane;

const float density = 0.0035;
const float gradient = 5.0;

void main(void) {

	vec4 worldPosition = transformationMatrix * vec4(position, 1.0);

	gl_ClipDistance[0] = dot(worldPosition, clipPlane);

	vec4 positionRelativeToCam = viewMatrix * worldPosition;
	gl_Position =  projectionMatrix * positionRelativeToCam;
	pass_textureCoords = (textureCoords / numberOfRows) + offset;

	vec3 actualNormal = normal;

	if(useFakeLighting > 0.5) {
	
		actualNormal = vec3(0.0, 1.0, 0.0);
	
	}

	surfaceNormal = (transformationMatrix * vec4(actualNormal, 0.0)).xyz;
	vec3 normSurfaceNormal = normalize(surfaceNormal);

	for(int i = 0; i < 4; i++) {

		toLightVector[i] = lightPosition[i] - worldPosition.xyz;

	}

	toCameraVector = (inverse(viewMatrix) * vec4(0.0, 0.0, 0.0, 1.0)).xyz - worldPosition.xyz;

	float distance = length(positionRelativeToCam.xyz);
	visibility = exp(-pow((distance * density), gradient));
	visibility = clamp(visibility, 0.0, 1.0);

	pass_tangent = normalize(mat3(transformationMatrix) * tangent);
	bitangent = normalize(cross(normSurfaceNormal, pass_tangent));

}