#version 400 core

in vec4 clipSpace;
in vec2 textureCoords;

//copied from entityShader for Fresnell shading
in vec3 toCameraVector;
vec3 waterNormal = vec3(0, 1, 0);

out vec4 out_Color;

uniform sampler2D reflectionTexture;
uniform sampler2D refractionTexture;
uniform sampler2D dudvMap;

uniform float moveFactor;

const float waveStrength = 0.02;

void main(void) {

	vec2 normDeviceCoords = (clipSpace.xy / clipSpace.w) / 2.0 + 0.5;
	vec2 refractTexCoords = normDeviceCoords;
	vec2 reflectTexCoords = vec2(normDeviceCoords.x, -normDeviceCoords.y);

	vec2 distortion1 = (texture(dudvMap, vec2(textureCoords.x + moveFactor, textureCoords.y)).rg * 2.0 - 1.0) * waveStrength;
	vec2 distortion2 = (texture(dudvMap, vec2(-textureCoords.x + moveFactor, textureCoords.y + moveFactor)).rg * 2.0 - 1.0) * waveStrength;
	vec2 socialDistortion = distortion1 + distortion2;

	refractTexCoords += socialDistortion;
	refractTexCoords = clamp(refractTexCoords, 0.001, 0.999);

	reflectTexCoords += socialDistortion;
	reflectTexCoords.x = clamp(reflectTexCoords.x, 0.001, 0.999);
	reflectTexCoords.y = clamp(reflectTexCoords.y, -0.999, -0.001);

	vec4 reflectColour = texture(reflectionTexture, reflectTexCoords);
	vec4 refractColour = texture(refractionTexture, refractTexCoords);

    //copied from entityShader for Fresnell shading
    vec3 unitVectorToCamera = normalize(toCameraVector);
    float mixingCoefficient = max(0, dot(waterNormal, unitVectorToCamera));

	out_Color = mix(reflectColour, refractColour , pow(mixingCoefficient, 5));
	out_Color = mix(out_Color, vec4(0.0, 0.3, 0.5, 1.0), 0.2);

}
